# -*- coding: UTF-8 -*-
import pytest
import os


@pytest.fixture
def SERVER():
    return os.environ['SUPLA_SERVER']

@pytest.fixture
def PERSONAL_ACCESS_TOKEN():
    return os.environ['SUPLA_PERSONAL_ACCESS_TOKEN']

@pytest.fixture
def SHUTTER_ID():
    return int(os.environ['SUPLA_SHUTTER_ID'])

@pytest.fixture
async def api(SERVER, PERSONAL_ACCESS_TOKEN, loop):
    from asyncpysupla import SuplaAPI

    api = SuplaAPI(
        server=SERVER,
        personal_access_token=PERSONAL_ACCESS_TOKEN
    )

    yield api

    await api.close()



# -*- coding: UTF-8 -*-
import logging
from urllib.parse import urljoin


import aiohttp


log = logging.getLogger(__name__)


class SuplaAPI:

    def __init__(
        self,
        server: str,
        personal_access_token: str,
        session: aiohttp.ClientSession = None,
    ):
        self.server = server
        self.base_url = f"https://{server}/api/v2.3.0/"
        self.personal_access_token = personal_access_token
        self.session = session if session else aiohttp.ClientSession()

    async def close(self):
        if self.session is not None:
            await self.session.close()
            self.session = None

    async def request(self, method: str, path: str, **kwargs) -> aiohttp.ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)

        headers["Authorization"] = f"Bearer {self.personal_access_token}"

        return await self.session.request(
            method, urljoin(self.base_url, path), **kwargs, headers=headers,
        )

    async def get_server_info(self):
        resp = await self.request("get", "server-info")
        return await resp.json()

    async def get_channels(self, include=None, func=None):
        params = {}

        if func is not None:
            params["function"] = ",".join(func)

        if include is not None:
            params["include"] = ",".join(include)

        resp = await self.request("get", "channels", params=params)
        return await resp.json()

    async def get_channel(self, channel_id, include=None):
        params = {}

        if include is not None:
            params["include"] = ",".join(include)

        resp = await self.request("get", f"channels/{channel_id}", params=params)
        return await resp.json()

    async def execute_action(self, channel_id, action, **add_pars):
        params = dict(
            action=action
        )
        params.update(add_pars)

        resp = await self.request("patch", f"channels/{channel_id}", json=params)
        log.debug("Action '%s' response on channel '%s': %s", action, channel_id, resp.text)
        assert 200 < resp.status < 299



# -*- coding: UTF-8 -*-
import pathlib
from setuptools import find_packages, setup


HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
VERSION = '0.0.5'

setup(
    name='asyncpysupla',
    description="Simple Supla's OpenAPI async wrapper",
    long_description=README,
    long_description_content_type="text/markdown",
    author='Michal Wegrzynek',
    author_email='michal.wegrzynek@malloc.com.pl',
    url='https://github.com/mwegrzynek/asyncpysupla',
    license='Apache License',
    version=VERSION,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Libraries',
        'Topic :: Home Automation'
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'aiohttp'
    ]
)


'''
Channel API testing
'''
import pytest


@pytest.mark.asyncio
async def test_list_all_channels(api):
    channels = await api.get_channels()
    assert len(channels) > 0
    assert channels[0]['id'] > 0

@pytest.mark.asyncio
async def test_list_only_shutters(api):
    channels = await api.get_channels(func=['CONTROLLINGTHEROLLERSHUTTER'])

    assert len(channels) > 0
    for chan in channels:
        assert chan['function']['name'] == 'CONTROLLINGTHEROLLERSHUTTER'

@pytest.mark.asyncio
async def test_list_only_shutters_and_light_switches(api):
    channels = await api.get_channels(func=['CONTROLLINGTHEROLLERSHUTTER', 'LIGHTSWITCH'])

    assert len(channels) > 0
    for chan in channels:
        assert chan['function']['name'] in ('LIGHTSWITCH', 'CONTROLLINGTHEROLLERSHUTTER')

@pytest.mark.asyncio
async def test_get_shutter_info(api, SHUTTER_ID):
    shutter = await api.get_channel(SHUTTER_ID)
    assert shutter['function']['name'] == 'CONTROLLINGTHEROLLERSHUTTER'

@pytest.mark.asyncio
async def test_get_shutter_info_with_iodevice_and_state(api, SHUTTER_ID):
    shutter = await api.get_channel(SHUTTER_ID, include=('iodevice', 'state'))
    assert 'shut' in shutter['state']

@pytest.mark.asyncio
async def test_open_shutters(api, SHUTTER_ID):
    await api.execute_action(SHUTTER_ID, 'REVEAL')

@pytest.mark.asyncio
async def test_close_shutters(api, SHUTTER_ID):
    await api.execute_action(SHUTTER_ID, 'SHUT')

@pytest.mark.asyncio
async def test_get_channels_with_devices_and_state(api):
    channels = await api.get_channels(include=["iodevice", "state", "connected"])

    assert 'connected' in channels[0]['state']